unit DataTypes;

interface

type
  PRGB = ^TRGB;
  TRGB = record
    b,g,r : byte;
  end;

  PComplex = ^TComplex;
  TComplex = record
    Real : single;
    Image : single;
  end;

  TCalcMode = (cmNormal, cmParallel);

  //  ""
  TParams = class(TObject)
  private
    //  
    FBetta : single;
    FDiameter : single;
    FWaveLength : single;
    FActWaveLength : single;
    FPointS, FPointDeltaX, FPointDeltaY : single;
    FDistance : single;
    FScreenSide : single;
    FScreenLength : single;
    FDelta : single;
    FCirclePoints : integer;
    FRadiusDispersion : integer;
    FCalcMode : TCalcMode;

    //  
    FNSamples : integer;
    FNSamplesScrSide : integer;
    FNSamplesScrLen : integer;

    procedure SetDiameter(Value : single);
    procedure SetDelta(Value : single);
    procedure SetScreenLength(Value : single);
    procedure SetScreenSide(Value : single);

    procedure LoadDefaults;
    procedure ReCalcDependent;

  public
    property Betta : single read FBetta write FBetta;
    property Diameter : single read FDiameter write SetDiameter;
    property WaveLength : single read FWaveLength write FWaveLength;
    property ActWaveLength : single read FActWaveLength write FActWaveLength;
    property PointS : single read FPointS write FPointS;
    property PointDeltaX : single read FPointDeltaX write FPointDeltaX;
    property PointDeltaY : single read FPointDeltaY write FPointDeltaY;
    property Distance : single read FDistance write FDistance;
    property ScreenSide : single read FScreenSide write SetScreenSide;
    property ScreenLength : single read FScreenLength write SetScreenLength;
    property Delta : single read FDelta write SetDelta;
    property CirclePoints : integer read FCirclePoints write FCirclePoints;
    property RadiusDispersion : integer read FRadiusDispersion write FRadiusDispersion;
    property CalcMode : TCalcMode read FCalcMode write FCalcMode;

    property NSamples : integer read FNSamples;
    property NSamplesScrLen : integer read FNSamplesScrLen;
    property NSamplesScrSide : integer read FNSamplesScrSide;

    procedure Assign(Source : TParams);
    function Differs(Source : TParams) : boolean;

    constructor Create;
  end;


implementation

uses
  Math;

// ========  "" ========
// -   
procedure TParams.SetDiameter(Value : single);
begin
  FDiameter := Value;
  ReCalcDependent;
end;

procedure TParams.SetDelta(Value : single);
begin
  FDelta := Value;
  ReCalcDependent;
end;

procedure TParams.SetScreenLength(Value : single);
begin
  FScreenLength := Value;
  ReCalcDependent;
end;

procedure TParams.SetScreenSide(Value : single);
begin
  FScreenSide := Value;
  ReCalcDependent;
end;

// -  
procedure TParams.LoadDefaults;
begin
  //     
  FBetta := Pi / 20;
  FDiameter := 15;
  FWaveLength := 0.5;
  FActWaveLength := 0.5;
  FPointS := 50;
  FPointDeltaX := 0;
  FPointDeltaY := 0;
  FDistance := 30;
  FScreenLength := 50;
  FScreenSide := 10;
  FDelta := 0.15;
  FCirclePoints := 72;
  FRadiusDispersion := 0;
  FCalcMode := cmNormal;
  //  
  ReCalcDependent;
end;

procedure TParams.ReCalcDependent;
begin
  FNSamples := Round(Diameter / Delta) and $FFFFFFFE;
  FNSamplesScrLen := Round(ScreenLength / Delta) and $FFFFFFFE;
  FNSamplesScrSide := Round(ScreenSide / Delta) and $FFFFFFFE;
end;

// -  
procedure TParams.Assign(Source : TParams);
begin
  //   
  FBetta := Source.Betta;
  FDiameter := Source.Diameter;
  FWaveLength := Source.WaveLength;
  FActWaveLength := Source.ActWaveLength;
  FPointS := Source.PointS;
  FPointDeltaX := Source.PointDeltaX;
  FPointDeltaY := Source.PointDeltaY;
  FDistance := Source.Distance;
  FScreenLength := Source.ScreenLength;
  FScreenSide := Source.ScreenSide;
  FCirclePoints := Source.CirclePoints;
  FRadiusDispersion := Source.RadiusDispersion;
  FDelta := Source.Delta;
  FCalcMode := Source.CalcMode;
  //  
  ReCalcDependent;
end;

function TParams.Differs(Source : TParams) : boolean;
begin
  Result :=
    (FBetta <> Source.Betta) or
    (FDiameter <> Source.Diameter) or
    (FWaveLength <> Source.WaveLength) or
    (FActWaveLength <> Source.ActWaveLength) or
    (FPointS <> Source.PointS) or
    (FPointDeltaX <> Source.PointDeltaX) or
    (FPointDeltaY <> Source.PointDeltaY) or
    (FDistance <> Source.Distance) or
    (FScreenLength <> Source.ScreenLength) or
    (FScreenSide <> Source.ScreenSide) or
    (FCirclePoints <> Source.CirclePoints) or
    (FRadiusDispersion <> Source.RadiusDispersion) or
    (FDelta <> Source.Delta) or
    (FCalcMode <> Source.CalcMode);
end;

constructor TParams.Create;
begin
  inherited Create;
  LoadDefaults;
end;

end.
