unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ComCtrls, Menus, ToolWin, ActnList, ImgList, DataTypes,
  StdCtrls, Buttons, jpeg;

type
  TFormMain = class(TForm)
    ActionList: TActionList;
    ToolBar: TToolBar;
    StatusBar: TStatusBar;
    SBDifrItem: TScrollBox;
    SBDiagram: TScrollBox;
    ImageList: TImageList;
    ImgDifrItem: TImage;
    ImgDiagram: TImage;
    ActParams: TAction;
    ActionSynth: TAction;
    TBProcessParams: TToolButton;
    TBSynth: TToolButton;
    ActSaveDiagram: TAction;
    SaveDialog: TSaveDialog;
    PMDiagram: TPopupMenu;
    MISaveDiagram: TMenuItem;
    ActSaveDI: TAction;
    PMDifrItem: TPopupMenu;
    N1: TMenuItem;
    SpeedButton1: TSpeedButton;
    Image1: TImage;
    SpeedButton2: TSpeedButton;
    Image2: TImage;
    ColorDialog1: TColorDialog;
    Button1: TButton;
    Button2: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Splitter: TSplitter;
    Label3: TLabel;
    Label4: TLabel;
    SpeedButton3: TSpeedButton;
    SaveDialog1: TSaveDialog;
    PopupMenu1: TPopupMenu;
    SaveImage1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure SBDifrItemResize(Sender: TObject);
    procedure ImgDifrItemMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ImgDiagramMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ActParamsExecute(Sender: TObject);
    procedure ActionSynthExecute(Sender: TObject);
    procedure SBDiagramResize(Sender: TObject);
    procedure ActSaveDiagramExecute(Sender: TObject);
    procedure ActSaveDiagramUpdate(Sender: TObject);
    procedure ActSaveDIExecute(Sender: TObject);
    procedure ImgDiagramMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure Saveimage1Click(Sender: TObject);
    procedure SaveDialog1CanClose(Sender: TObject; var CanClose: Boolean);
  private
    Params : TParams;

    S : array of TComplex;
    SPresent : boolean;
    DiagramPresent : boolean;
    GraphColorIdx : integer;
    GraphColorOverriden : boolean;

    procedure DifrItemUpdatePosition;
    procedure DifrItemUpdateSize;
    procedure DifrItemInit;
    procedure DifrItemRender;

    procedure DiagramUpdatePosition;
    procedure DiagramUpdateSize;
    procedure DiagramInit;
    procedure DiagramRender;

    procedure SGenerateNormal;
    procedure SGenerateParallel;
    procedure SGenerate;
    procedure SFree;

  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;
  wad,w:boolean;
implementation

uses
  Math, ParamsEdit, Unit2;

const
  METAL_COLOR = 0;
  NONMETAL_COLOR = $00d6be16;

{$R *.dfm}

// ========  ========Calculation

procedure TFormMain.SGenerateNormal;
var
  PhaseMatrix : array of TComplex;
  pValue : PComplex;
  RowIdx,ColIdx : integer;
  Cf, Arg : single;
  dY2, dZ2 : single;
  R : single;
  Delta : single;
  NSamples : integer;
  NSamplesScr : integer;
  NSamplesPhase : integer;
  NSamplesConv : integer;

  DRowIdx,DColIdx,SRowIdx,SColIdx : integer;
  pDValue : PRGB;
  pSValue : PComplex;
  SLength : integer;
  ItemCnt : integer;
  Ph1Real, Ph1Image : single;
  Ph2Real, Ph2Image : single;
  PhMatrixOffset : integer;
begin
  //     Cash of some parameters
  Delta := Params.Delta;
  NSamples := Params.NSamples;
  NSamplesScr := Params.NSamplesScrSide;
  NSamplesPhase := (NSamples + NSamplesScr) div 2;
  NSamplesConv := NSamples div 2 - NSamplesScr div 2;

  //     array of phase
  SetLength(PhaseMatrix,sqr(NSamplesPhase));
  Cf := 2 * Pi / Params.ActWaveLength;
  dZ2 := sqr(Params.Distance);
  pValue := @PhaseMatrix[0];
  RowIdx := 0;

  // -    array of phase download
  while RowIdx < NSamplesPhase do begin
    dY2 := sqr(RowIdx * Delta);
    ColIdx := 0;
    while ColIdx < NSamplesPhase do begin
      R := sqrt(sqr(ColIdx*Delta) + dY2 + dZ2);
      Arg := Cf * R;
      with pValue^ do begin
        Real := cos(Arg) / R;
        Image := sin(Arg) / R;
      end;
      Inc(ColIdx);
      Inc(pValue);
    end;
    Inc(RowIdx);
  end;

  //      /  Clearing amplitude-phase array
  SLength := sqr(NSamplesScr);
  SetLength(S,SLength);
  ItemCnt := SLength;
  pSValue := @S[0];
  while ItemCnt > 0 do begin
    with pSValue^ do begin
      Real := 0;
      Image := 0;
    end;
    Dec(ItemCnt);
    Inc(pSValue);
  end;
  SPresent := true;

  //   ( ) ampl-phase array
  dZ2 := sqr(Params.PointS);

  // -       step on all point of image
  DRowIdx := 0;
  while DRowIdx < NSamples do begin
    StatusBar.Panels[0].Text := 'Calculation (' + IntToStr(DRowIdx+1) + ' of ' + IntToStr(NSamples) + ')';
    Application.ProcessMessages;
    pDValue := ImgDifrItem.Picture.Bitmap.ScanLine[DRowIdx];
    DColIdx := 0;
    while DColIdx < NSamples do begin
      if pDValue^.r = 0 then begin

        //        phase length from
        // sources
        R := sqrt(sqr((DColIdx - NSamples div 2) * Delta - Params.PointDeltaX) + sqr((DRowIdx - NSamples div 2) * Delta - Params.PointDeltaY) + dZ2);
        Arg := Cf * R;
        Ph1Real := cos(Arg) / R;
        Ph1Image := sin(Arg) / R;

      //      cycle for all points of screen
        pSValue := @S[0];
        SRowIdx := 0;
        while SRowIdx < NSamplesScr do begin
          SColIdx := 0;
          PhMatrixOffset := Abs(DRowIdx - SRowIdx - NSamplesConv) * (NSamplesPhase);
          while SColIdx < NSamplesScr do begin
            with PhaseMatrix[phMatrixOffset + Abs(DColIdx - SColIdx - NSamplesConv)] do begin
              Ph2Real := Real;
              Ph2Image := Image;
            end;
            with pSValue^ do begin
              Real := Real + (Ph1Real*Ph2Real - Ph1Image*Ph2Image);
              Image := Image + (Ph1Real*Ph2Image + Ph1Image*Ph2Real);
            end;
            Inc(SColIdx);
            Inc(pSValue);
          end;
          Inc(SRowIdx);
        end;
      end;
      Inc(DColIdx);
      Inc(pDValue);
    end;
    Inc(DRowIdx);
  end;
end;

procedure TFormMain.SGenerateParallel;
var
  DRowIdx,DColIdx,SRowIdx,SColIdx : integer;
  pDValue : PRGB;
  pSValue : PComplex;
  SLength : integer;
  ItemCnt : integer;
  Ph1Real, Ph1Image : single;
  Ph2Real, Ph2Image : single;

  Cf, Arg : single;
  dZ12 : single;
  S2 : single;
  dY22, dZ22 : single;
  R : single;
  Delta : single;
  NSamples : integer;
  NSamples_2 : integer;
  NSamplesConvX : integer;
  NSamplesConvY : integer;
  WidthSamples, HeightSamples : integer;
begin
  //   
  Delta := Params.Delta;
  NSamples := Params.NSamples;
  WidthSamples := Params.NSamplesScrSide;
  HeightSamples := Params.NSamplesScrLen;
  NSamplesConvX := NSamples div 2 - WidthSamples div 2;

  //      / 
  SLength := WidthSamples * HeightSamples;
  SetLength(S,SLength);
  ItemCnt := SLength;
  pSValue := @S[0];
  while ItemCnt > 0 do begin
    with pSValue^ do begin
      Real := 0;
      Image := 0;
    end;
    Dec(ItemCnt);
    Inc(pSValue);
  end;
  SPresent := true;

  //   ( )
  Cf := 2 * Pi / Params.ActWaveLength * Delta;
  dZ12 := sqr(Params.PointS / Delta);
  NSamples_2 := NSamples div 2;
  S2 := (Params.Distance + Params.ScreenLength / 2) / Delta;
  // -      
  DRowIdx := 0;
  while DRowIdx < NSamples do begin
    StatusBar.Panels[0].Text := 'Calculation (' + IntToStr(DRowIdx+1) + ' of ' + IntToStr(NSamples) + ')';
    Application.ProcessMessages;
    pDValue := ImgDifrItem.Picture.Bitmap.ScanLine[DRowIdx];
    DColIdx := 0;
    while DColIdx < NSamples do begin
      if pDValue^.r = 0 then begin
        // -       
        R := sqrt(sqr((DColIdx - NSamples_2) - Params.PointDeltaX / Delta) + sqr((DRowIdx - NSamples_2) - Params.PointDeltaY / Delta) + dZ12);
        Arg := Cf * R;
        Ph1Real := cos(Arg) / R;
        Ph1Image := sin(Arg) / R;
        // -       
        dY22 := sqr(DRowIdx - NSamples_2);
        //     
        pSValue := @S[0];
        SRowIdx := 0;
        while SRowIdx < HeightSamples do begin
          dZ22 := sqr(S2 - SRowIdx);
          SColIdx := 0;
          while SColIdx < WidthSamples do begin
            // -       screen phase
            R := sqrt(sqr(DColIdx - SColIdx - NSamplesConvX) + dY22 + dZ22);
            Arg := Cf * R;
            Ph2Real := cos(Arg) / R;
            Ph2Image := sin(Arg) / R;
            with pSValue^ do begin
              Real := Real + (Ph1Real*Ph2Real - Ph1Image*Ph2Image);
              Image := Image + (Ph1Real*Ph2Image + Ph1Image*Ph2Real);
            end;
            Inc(SColIdx);
            Inc(pSValue);
          end;
          Inc(SRowIdx);
        end;
      end;
      Inc(DColIdx);
      Inc(pDValue);
    end;
    Inc(DRowIdx);
  end;

end;

procedure TFormMain.SGenerate;
begin
  Application.ProcessMessages;
  if Params.CalcMode = cmNormal then SGenerateNormal
  else SGenerateParallel;
  StatusBar.Panels[0].Text := '';
end;

procedure TFormMain.SFree;
begin
  SetLength(S,0);
  SPresent := false;
end;

// ========   ========
// -  /   ========
// Images procedure
procedure TFormMain.DifrItemUpdatePosition;
var
  Coord : integer;
begin
  Coord := (SBDifrItem.ClientWidth - ImgDifrItem.Width) div 2;
  if Coord >= 0 then begin
    SBDifrItem.HorzScrollBar.Position := 0;
    ImgDifrItem.Left := Coord;
  end else
    ImgDifrItem.Left := -SBDifrItem.HorzScrollBar.Position;

  Coord := (SBDifrItem.ClientHeight - ImgDifrItem.Height) div 2;
  if Coord >= 0 then begin
    SBDifrItem.VertScrollBar.Position := 0;
    ImgDifrItem.Top := Coord
  end else
    ImgDifrItem.Top := -SBDifrItem.VertScrollBar.Position;
end;

procedure TFormMain.DifrItemUpdateSize;
begin
  ImgDifrItem.Height := Params.NSamples;
  ImgDifrItem.Width := Params.NSamples;

  DifrItemUpdatePosition;
end;

procedure TFormMain.DifrItemInit;
begin
  DifrItemUpdateSize;
  DifrItemRender;
end;

procedure TFormMain.DifrItemRender;
var
  X0,Y0 : integer;
  k : integer;
  sin_b, cos_b : single;
  l : single;
  wl : single;
  c : single;
  r : single;
  r_smp : single;
  Rmax : single;
  PokrMetal : boolean;
  c1 : integer;
  Points : array of TPoint;
begin
  ImgDifrItem.Picture.Bitmap.PixelFormat := pf24Bit;
  ImgDifrItem.Picture.Bitmap.Height := Params.NSamples;
  ImgDifrItem.Picture.Bitmap.Width := Params.NSamples;
  //   ( k)
  Rmax := Params.Diameter / 2;
  l := Params.Diameter;
  wl := Params.WaveLength;
  sin_b := sin(Params.Betta);
  cos_b := cos(Params.Betta);
  k := 1;
  try
    while True do begin
      c := l + k * wl;
//  
//    r := c * sin_b / sqr(cos_b) - sqrt(sqr(c * sin_b / sqr(cos_b)) - (2*k*l*wl + sqr(k*wl)) / sqr(cos_b));
      r := -c * sin_b / sqr(cos_b) + sqrt(sqr(c)-l*l*cos_b*cos_b) / sqr(cos_b);
      if r > Rmax then break;
      Inc(k);
    end;

  except
    MessageDlg('Error: the radii is ( k=' + IntToStr(k) + ').',mtError,[mbOK],0);
  end;
  Dec(k);
  with ImgDifrItem.Picture.Bitmap do begin
    // 
    Canvas.Brush.Color := METAL_COLOR;
    Canvas.FillRect(Rect(0,0,Width,Height));
    // 
    X0 := Round(Params.Diameter / 2 / Params.Delta);
    Y0 := Round(Params.Diameter / 2 / Params.Delta);

    PokrMetal := false;
    while k > 0 do begin
      if PokrMetal then Canvas.Brush.Color := METAL_COLOR
      else Canvas.Brush.Color := NONMETAL_COLOR;
      c := l + k * wl;
//  
//    r := c * sin_b / sqr(cos_b) - sqrt(sqr(c * sin_b / sqr(cos_b)) - (2*k*l*wl + sqr(k*wl)) / sqr(cos_b));
      r := -c * sin_b / sqr(cos_b) + sqrt(sqr(c)-l*l*cos_b*cos_b) / sqr(cos_b);
      r_smp := r / Params.Delta;
      SetLength(Points,Params.CirclePoints + 1);
      for c1 := 0 to Params.CirclePoints - 1 do
      with Points[c1] do begin
        X := X0 + Trunc(r_smp * cos(2 * Pi * c1 / Params.CirclePoints) * (1 + Random(Params.RadiusDispersion)/100));
        Y := Y0 + Trunc(r_smp * sin(2 * Pi * c1 / Params.CirclePoints) * (1 + Random(Params.RadiusDispersion)/100));
      end;
      Points[Params.CirclePoints] := Points[0];
      Canvas.Polygon(Points);
      //Canvas.Ellipse(Round(X0-r_smp), Round(Y0-r_smp), Round(X0+r_smp), Round(Y0+r_smp));
      PokrMetal := not PokrMetal;
      Dec(k);
    end;
  end;
end;

// -  / 
procedure TFormMain.DiagramUpdatePosition;
var
  Coord : integer;
begin
  Coord := (SBDiagram.ClientWidth - ImgDiagram.Width) div 2;
  if Coord >= 0 then begin
    SBDiagram.HorzScrollBar.Position := 0;
    ImgDiagram.Left := Coord;
  end else
    ImgDiagram.Left := -SBDiagram.HorzScrollBar.Position;
  Coord := (SBDiagram.ClientHeight - ImgDiagram.Height) div 2;
  if Coord >= 0 then begin
    SBDiagram.VertScrollBar.Position := 0;
    ImgDiagram.Top := Coord;
  end else
    ImgDiagram.Top := -SBDiagram.VertScrollBar.Position;
end;

procedure TFormMain.DiagramUpdateSize;
begin
  ImgDiagram.Width := Params.NSamplesScrSide;
  if Params.CalcMode = cmNormal then
    ImgDiagram.Height := Params.NSamplesScrSide
  else
    ImgDiagram.Height := Params.NSamplesScrLen;
  DiagramUpdatePosition;
end;

procedure TFormMain.DiagramInit;
begin
  with ImgDiagram do begin
    Width := 0;
    Height := 0;
    Picture.Bitmap.Width := 0;
    Picture.Bitmap.Height := 0;
  end;
  DiagramPresent := false;
end;

procedure TFormMain.DiagramRender;
var
  AbsS : array of single;
  WidthSamples, HeightSamples : integer;
  SLength : integer;
  ItemCnt : integer;
  pSValue : PComplex;
  pValue : PSingle;
  Value : single;
  Max : single;
  RowIdx, ColCnt : integer;
  pPixel : PRGB;
  Color,o : integer;
begin
  SGenerate;
  StatusBar.Panels[0].Text := 'Rendering...';
  Application.ProcessMessages;

  WidthSamples := Params.NSamplesScrSide;
  if Params.CalcMode = cmNormal then HeightSamples := Params.NSamplesScrSide
  else HeightSamples := Params.NSamplesScrLen;
  SLength := WidthSamples * HeightSamples;
  SetLength(AbsS,SLength);

  //      
  pSValue := @S[0];
  pValue := @AbsS[0];
  ItemCnt := SLength;
  Max := 0;
  while ItemCnt > 0 do begin
    with pSValue^ do
      Value := sqrt(sqr(Real) + sqr(Image));
    if Value > Max then Max := Value;
    pValue^ := Value;
    Inc(pSValue);
    Inc(pValue);
    Dec(ItemCnt);
  end;
  if Max = 0 then Max := 1;

  ImgDiagram.Picture.Bitmap.PixelFormat := pf24Bit;
  ImgDiagram.Picture.Bitmap.Width := WidthSamples;
  ImgDiagram.Picture.Bitmap.Height := HeightSamples;

  pValue := @AbsS[0];
  RowIdx := 0;
  while RowIdx < HeightSamples do begin
    pPixel := ImgDiagram.Picture.Bitmap.ScanLine[RowIdx];
    ColCnt := WidthSamples - 1;
    while ColCnt >= 0 do begin
      Color := trunc(pValue^ / Max * 255);
      if Color > 255 then Color := 255;
      with pPixel^ do begin
        b:=0;
        g:=color;
        r:=0;
        end;
      Inc(pValue);
      Inc(pPixel);
      Dec(ColCnt);
    end;
    Inc(RowIdx);
  end;
  DiagramUpdateSize;
  ImgDiagram.Invalidate;
  DiagramPresent := true;
end;


// ========   ========
// - 
procedure TFormMain.FormCreate(Sender: TObject);
begin
  GraphColorIdx := 1;
  wad:=true;
  w:=true;
  Params := TParams.Create;
  //   /
  DifrItemInit;
  DiagramInit;
end;

// -    
procedure TFormMain.SBDifrItemResize(Sender: TObject);
begin
  DifrItemUpdatePosition;
end;

procedure TFormMain.ImgDifrItemMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
  Xmm,Ymm : single;
begin
  Xmm := (X - Params.NSamples / 2) * Params.Delta;
  Ymm := (Y - Params.NSamples / 2) * Params.Delta;
  StatusBar.Panels[1].Text := Format('(%.2f;%.2f) R = %.2f [mm]',[Xmm,Ymm,sqrt(sqr(Xmm)+sqr(Ymm))]);
end;

// -   
procedure TFormMain.SBDiagramResize(Sender: TObject);
begin
  DiagramUpdateSize;
end;

procedure TFormMain.ImgDiagramMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
label3.Caption:=inttostr(x);
label4.Caption:=inttostr(y);
end;


// ========  ========
procedure TFormMain.ActParamsExecute(Sender: TObject);
begin
  if FormParamsEdit.Execute(Params) then begin
    SFree;
    DifrItemUpdateSize;
    DifrItemRender;
    DiagramInit;
  end;
end;

procedure TFormMain.ActionSynthExecute(Sender: TObject);
 var
  c1: integer;
  day,mon,year:word;
  S : string;
  dt: tdatetime;
  g:integer;
 begin
  dt:=now;
  g:=0;
  DecodeDate(dt,year,mon,day);
{      ,     "Trial period"}
//  if (year>2003) or ((year=2003) and (mon>4) and (day>13)) then begin c1:=1 div g;label1.Caption:=inttostr(c1);end;
  //    (      )
  speedbutton2.Glyph.LoadFromFile('button.bmp');
  w:=true;
  image2.Visible:=false;
  imgdiagram.Visible:=true;

  DifrItemRender;
  DiagramRender;
  StatusBar.Panels[0].Text := 'Ready';
  StatusBar.Panels[2].Text := '';
end;

procedure TFormMain.ActSaveDiagramExecute(Sender: TObject);
var
  F : TextFile;
  SLength : integer;
  ItemIndex : integer;
begin
  if not SaveDialog.Execute then exit;
  if Params.CalcMode = cmNormal then SLength := sqr(Params.NSamplesScrSide)
  else  SLength := sqr(Params.NSamplesScrSide * Params.NSamplesScrLen);
  AssignFile(F,SaveDialog.FileName);
  ReWrite(F);
  for ItemIndex := 0 to SLength - 1 do
  with S[ItemIndex] do begin
    WriteLn(F,Real);
    WriteLn(F,Image);
  end;
  CloseFile(F);
end;

procedure TFormMain.ActSaveDiagramUpdate(Sender: TObject);
begin
  ActSaveDiagram.Enabled := SPresent;
end;

procedure TFormMain.ActSaveDIExecute(Sender: TObject);
var
  F : TextFile;
  RowIdx, ColIdx : integer;
  pPixel : PRGB;
begin
  if not SaveDialog.Execute then exit;
  AssignFile(F,SaveDialog.FileName);
  ReWrite(F);
  RowIdx := Params.NSamples - 1;
  while RowIdx >= 0 do begin
    pPixel := ImgDifrItem.Picture.Bitmap.ScanLine[RowIdx];
    ColIdx := Params.NSamples - 1;
    while ColIdx >= 0 do begin
      if pPixel.r <> 0 then WriteLn(F,1)
      else WriteLn(F,0);
      Inc(pPixel);
      Dec(ColIdx);
    end;
    Dec(RowIdx);
  end;
  CloseFile(F);
end;

procedure TFormMain.ImgDiagramMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var n,m:integer;
    ert:real;
    Col : integer;
begin
  if GraphColorOverriden then
    Col:=colordialog1.Color
  else begin
    Col := 0;
    if (GraphColorIdx and $01) > 0 then Col := Col + 192;
    if (GraphColorIdx and $02) > 0 then Col := Col + 192 shl 8;
    if (GraphColorIdx and $04) > 0 then Col := Col + 192 shl 16;
    Inc(GraphColorIdx);
    if GraphColorIdx = 7 then GraphColorIdx := 1;
  end;
  GraphColorOverriden := False;

image1.Canvas.Pen.Color:=clblack;
image1.Canvas.MoveTo(20,265);
image1.Canvas.LineTo(280,265);
image1.Canvas.MoveTo(25,270);
image1.Canvas.LineTo(25,0);
for n:=0 to 10 do
    image1.Canvas.TextOut(20+n*25,270,inttostr(n*10));
for n:=1 to 13 do
    image1.Canvas.TextOut(2,265-n*20,inttostr(n*20));
ert:=0;

if wad then begin
image1.Canvas.MoveTo(26,265-imgdiagram.Canvas.Pixels[0,y] div 256);
for n:=1 to imgdiagram.width-1 do
    begin
    ert:=ert+250/imgdiagram.width;
    m:=round(ert);
    image1.Canvas.Pen.Color:=col;
    image1.Canvas.LineTo(m+25,265-imgdiagram.Canvas.Pixels[n,y] div 256);
    end;
end
else
begin
image1.Canvas.MoveTo(26,265-imgdiagram.Canvas.Pixels[x,0] div 256);
for n:=1 to imgdiagram.height-1 do
    begin
    ert:=ert+250/imgdiagram.height;
    m:=round(ert);
    image1.Canvas.Pen.Color:=col;
    image1.Canvas.LineTo(m+25,265-imgdiagram.Canvas.Pixels[x,n] div 256);
    end;
end;

end;

procedure TFormMain.SpeedButton1Click(Sender: TObject);
begin
if wad then
    begin
    speedbutton1.Glyph.LoadFromFile('b2.bmp');
    wad:=false;
    end
    else
    begin
    speedbutton1.Glyph.LoadFromFile('b1.bmp');
    wad:=true;
    end;
end;

procedure TFormMain.SpeedButton2Click(Sender: TObject);
var i,j,o,p,l: integer;
begin
if w=true then begin
image2.Left:=imgdiagram.Left;
image2.Top:=imgdiagram.Top;
image2.Height:=imgdiagram.Height;
image2.Width:=imgdiagram.Width;
For i:=0 to image2.Width  do
 for j:=0  to image2.Height do
  begin
    o:=imgdiagram.Canvas.Pixels[i,j] div 256;
    o:=256-(o-(o div 256*256));
    if (o>=0) and (o<42) then p:=(255-round(o/42*255))*256*256+255;
    if (o>=42) and (o<84) then p:=(round((o-42)/42*256))*256+255;
    if (o>=84) and (o<126) then p:=round(255-((o-84)/42*256))+255*256;
    if (o>=126) and (o<168) then p:=round((o-126)/42*255)*256*256+255*256;
    if (o>=168) and (o<210) then p:=255*256*256+(255-round((o-168)/42*255))*256;
    if (o>=210) then p:=(255-round((o-210)/60*255))*256*256;
    image2.Canvas.Pixels[i,j]:=p;
  End;
w:=false;
image2.Visible:=true;
imgdiagram.Visible:=false;
speedbutton2.Glyph.LoadFromFile('button2.bmp');
end
else
begin
image2.Visible:=false;
imgdiagram.Visible:=true;
speedbutton2.Glyph.LoadFromFile('button.bmp');
w:=true;
end;
end;

procedure TFormMain.Button2Click(Sender: TObject);
begin
  if colordialog1.Execute then GraphColorOverriden := True;
end;

procedure TFormMain.Button1Click(Sender: TObject);
var n:integer;
begin
image1.Canvas.Brush.Color:=clwhite;
image1.Canvas.Ellipse(-100,-100,400,400);
image1.Canvas.Pen.Color:=clblack;
image1.Canvas.MoveTo(20,265);
image1.Canvas.LineTo(280,265);
image1.Canvas.MoveTo(25,270);
image1.Canvas.LineTo(25,0);
for n:=1 to 12 do
    image1.Canvas.TextOut(2,265-n*20,inttostr(n*20));
end;

procedure TFormMain.SpeedButton3Click(Sender: TObject);
begin
form2.visible:=true;
end;

procedure TFormMain.Saveimage1Click(Sender: TObject);
begin
Savedialog1.execute
end;

procedure TFormMain.SaveDialog1CanClose(Sender: TObject;
  var CanClose: Boolean);
begin
image1.Picture.SaveToFile(savedialog1.filename);
end;

end.
